--[[
LuaX windowing system
library

variables (you can change):
*window_barsize - the width of the bar at the top of the window
variables (you need to set):
*cursor = {x=,y=,img=,visible=true,state="none"}

functions:
LuaX_regwindow(window)
*adds the window
LuaX_closewindow([window_id])
*closes the given window, optional ID denotes which window to close, else close current window
LuaX_getWindowById(window_id)
*returns the window number of the window with a matching id, else returns nil
LuaX_process()
*processes and displays the windows

window structure:
{title="", id="", x=0, y=0, width=200, height=100, bgcolor=Color.new(0,0,0),
 func=dosomething, Xlocked=0, Mlocked=0, border=1, barcolor=Color.new(200,200,200), barstyle=1,
 TextBox={type="textbox", x=0, y=0, color=Color.new(255,255,255), value="test", onClick=something}
}
func is the function ran constantly. make sure you call sleep() inside it if it loops
onClick is ran when the user clicks it.
note that you can pass 0 to not run a function
also note that you access elements with window.*, IE: window.x = 5

]]

function dp(x,y) --debug print
screen:clear()
screen:print(0,0,tostring(x),Color.new(255,255,255))
screen.flip()
screen.waitVblankStart(y or 1)
end

local function distance(x,y,x2,y2)
return math.floor(math.sqrt((x2-x)^2 + (y2-y)^2))
end

function _LuaX_ct(t,x,y,tx)
t:print(x+1,y,tx,Color.new(100,100,100))
t:print(x-1,y,tx,Color.new(100,100,100))
t:print(x,y+1,tx,Color.new(100,100,100))
t:print(x,y-1,tx,Color.new(100,100,100))
t:print(x,y,tx,Color.new(255,255,255))
end

local function sprint(x,y,text)
	screen:print(x-1,y,text)
	screen:print(x+1,y,text)
	screen:print(x,y-1,text)
	screen:print(x,y+1,text)
	screen:print(x,y,text,Color.new(255,255,255))
end

--ALL BELOW COMMENTED CODE is taken care of the enviroment manager
window_barsize = 16
--windows = {} --all windows
--window_co = {} --the coroutines that are ran for window_chg_atts[4]
--window_imgs = {} --buffer for the window bg image
--window_chg_atts = {} --compare to see if window attributes have changed
					-- {width,height,bgcolor,func,title,movex,movey,objects[][],rerender,border,barcolor,state[]}
					--  1     2      3       4    5     6     7     8			9		 10		11		 12
					
					--the objects[][] is the exact same, IE:
					--window_chg_atts[7].myButton.x
					--state[] holds the window states:
					--{state= ("max","maxing","min","mining"),increaments={incrementx,incrementy,x+,y,x,y+,x+,y+}
default_object_params = {type="unk",
						 x=10,
						 y=10,
						 color=Color.new(100,100,100),
						 value="",
						 redraw=false
						}
LuaX_desktop_img = nil
LuaX_desktop_redraw = false
LuaX_desktop_icons = {} --structure: {name, img, onClick, [x, y]}
LuaX_desktop_dclickcnt = 0
LuaX_desktop_dclickcnt_max = 20

window_min_num = 20 --minimize speed
LuaX_state = "none"
LuaX_windtab_type = 2 -- 1=MAC tabs, 2=Flip3D
LuaX_windtab_fade = 0
LuaX_windtab_d = 0
LuaX_windtab_i = 0
LuaX_windtab_done = 0
statusbar_type = 1 --1=MAC dock, 2=startbar/menu
--statusbar_objects = {} -- {name, id, icon, image, onclick}
statusbar_redraw = true
--
statusbar_1_height = 20
statusbar_1_menuwidth = 150
statusbar_1_y = 0
statusbar_1_active = false
statusbar_1_menuimg = nil
--
statusbar_2_icondim = 32
statusbar_2_height = 40
statusbar_2_active = false
statusbar_2_magnify = 1.5
statusbar_2_magnify_sen = 4
--

statusbar_img = nil

--

_LuaX_screen = screen

LuaX_cube_state = 0
LuaX_cube_focalLength = 240
LuaX_cube_rot = 0
LuaX_cube_bgx = 0
LuaX_cube__rot = 0

local function make3DPoint(x,y,z)
	local point = {}
	point.x = x * (240/2)
	point.y = -y * (136/2)
	point.z = z * (240/2)
	return point
end

LuaX_cube_pointsArray = {
make3DPoint(-1, 1,-1),
make3DPoint( 1, 1,-1),
make3DPoint( 1, 1, 1),
make3DPoint(-1, 1, 1),

make3DPoint(-1,-1,-1),
make3DPoint( 1,-1,-1),
make3DPoint( 1,-1, 1),
make3DPoint(-1,-1, 1),
}
--

closedwindow = false --was a window closed?
sleep = coroutine.yield
color_change = 20

local _LuaX_win_id = 0 --each window has an unique ID for minimizing/maximizing


oldcursor_x = 0
oldcursor_y = 0
c_movx = 0 --cursor momentum
c_movy = 0
cursor_f = 1.5
window_f = 1.15
_LuaX_windowOver = false

if not LuaX_default_icon then
	LuaX_default_icon = Image.createEmpty(10,10)
	LuaX_default_icon:clear(Color.new(192,192,192))
	LuaX_default_icon:fillRect(1,1,8,2,Color.new(0,0,255))
	LuaX_default_icon:fillRect(1,3,8,6,Color.new(128,128,128))
	LuaX_default_icon:fillRect(2,3,6,5,Color.new(255,255,255))
end

if not LuaX_default_thumb then
	LuaX_default_thumb = Image.createEmpty(statusbar_2_icondim,statusbar_2_icondim)
	LuaX_default_thumb:clear(Color.new(255,255,255))
	LuaX_default_thumb:print(0,0,"PROG")
end

function getwindowclick()
if statusbar_type == 2 then --make sure you didnt click the bar area
	if statusbar_2_active then
		if (cursor.x >= 240-statusbar_img:width()/2 and cursor.x <= (240-statusbar_img:width()/2)+statuswidth and cursor.y >= 272-statusbar_2_icondim*statusbar_2_magnify) == false then return false end
	else
		if cursor.x >= 240-statusbar_img:width()/2 and cursor.x <= (240-statusbar_img:width()/2)+statusbar_img:width() and cursor.y >= 272-statusbar_2_height then return false end
	end
elseif statusbar_type == 1 then
	if cursor.y >= 272-statusbar_1_height+statusbar_1_y then return false end
	if statusbar_1_active then
		if cursor.y >= 272-statusbar_1_height+statusbar_1_y-statusbar_1_menuimg:height()+3 and cursor.y <= 272-statusbar_1_height+statusbar_1_y+3 and cursor.x < statusbar_1_menuimg:width() then
			return false
		end
	end
end
for a=1,table.getn(windows) do
	window = windows[a]
	if window_chg_atts[a][12][1] == "max" and cursor.x >= window.x and cursor.x <= window.x+window.width
	   and cursor.y >= window.y-window_barsize and cursor.y <= window.y+window.height then
		return a
	end
end
return 0
end

local function makecolorlighter(lc,ln,lt)
local temp = lc:colors()
local tr = temp.r+ln
local tg = temp.g+ln
local tb = temp.b+ln
if not lt then
if tr > 255 then tr = 255 - (tr-255) end
if tg > 255 then tg = 255 - (tg-255) end
if tb > 255 then tb = 255 - (tb-255) end
end
return Color.new(tr,tg,tb,temp.a)
end

local function makecolordarker(lc,ln,lt)
local temp = lc:colors()
local tr = temp.r-ln
local tg = temp.g-ln
local tb = temp.b-ln
if not lt then
if tr < 0 then tr = math.abs(tr) end
if tg < 0 then tg = math.abs(tg) end
if tb < 0 then tb = math.abs(tb) end
end
return Color.new(tr,tg,tb,temp.a)
end

function LuaX_regwindow(wind)
wind.func = wind.func or 0
wind.border = wind.border or 1
wind.barstyle = wind.barstyle or 1
wind.barcolor = wind.barcolor or Color.new(50,50,50,240)
table.insert(windows,wind)
table.insert(window_chg_atts,{wind.width,wind.height,wind.bgcolor,wind.func,wind.title,0,0,{},true,wind.border,wind.barcolor,{"max",{},0,{},_LuaX_win_id}})
_LuaX_win_id = _LuaX_win_id + 1
table.foreach(wind,function(a,b)
					if type(wind[a]) == "table" then
						window_chg_atts[table.getn(window_chg_atts)][8][a] = {}
						table.foreach(default_object_params,function(c,d)
							window_chg_atts[table.getn(window_chg_atts)][8][a][c] = wind[a][c] or d
						end )
					end
				   end)
local temp = Image.createEmpty(wind.width,wind.height+window_barsize)
table.insert(window_imgs,temp)
if type(wind.func) == "function" then
	table.insert(window_co,coroutine.create(wind.func))
else
	table.insert(window_co,0)
end
end

function LuaX_closewindow(wnum)
table.remove(windows,wnum or _wnum)
table.remove(window_co,wnum or _wnum)
table.remove(window_imgs,wnum or _wnum)
table.remove(window_chg_atts,wnum or _wnum)
closedwindow = true
end

function LuaX_minimizewindow(wnum)
window_chg_atts[wnum or _wnum][12][1] = "mining"

--estimate the new icons location
local tempx = statusbar_img:width() + statusbar_2_icondim + 10
tempx = 240-tempx/2 + tempx
tempx = tempx - (statusbar_2_icondim+10)
local tempy = (272-statusbar_2_height) + (statusbar_2_height/2-statusbar_2_icondim/2)

_addx = statusbar_2_icondim-1
_addy = statusbar_2_icondim-1
if statusbar_type == 1 then --modify for start bar
	--get total apps
	local total = 0
	for a=1,table.getn(statusbar_objects) do
		if statusbar_objects[a][1] == "win" then
			total = total + 1
		end
	end
	tempx = 60+total*(80+2)
	tempy = 272-statusbar_1_height+statusbar_1_y+4
	_addx = 79
	_addy = 13
end

local temp = math.max(1,math.floor(distance(tempx,tempy,windows[wnum or _wnum].x,windows[wnum or _wnum].y-window_barsize)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][1] = (tempx - windows[wnum or _wnum].x) / temp --topleft
window_chg_atts[wnum or _wnum][12][2][2] = (tempy - (windows[wnum or _wnum].y-window_barsize)) / temp
window_chg_atts[wnum or _wnum][12][4][1] = temp --max loops

temp = math.max(1,math.floor(distance(tempx+_addx,tempy,windows[wnum or _wnum].x+windows[wnum or _wnum].width,windows[wnum or _wnum].y-window_barsize)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][3] = ((tempx+_addx) - (windows[wnum or _wnum].x+windows[wnum or _wnum].width)) / temp --topright
window_chg_atts[wnum or _wnum][12][2][4] = (tempy - (windows[wnum or _wnum].y-window_barsize)) / temp
window_chg_atts[wnum or _wnum][12][4][2] = temp

temp = math.max(1,math.floor(distance(tempx,tempy+_addy,windows[wnum or _wnum].x,windows[wnum or _wnum].y+windows[wnum or _wnum].height)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][5] = (tempx - windows[wnum or _wnum].x) / temp --bottomleft
window_chg_atts[wnum or _wnum][12][2][6] = ((tempy+_addy) - (windows[wnum or _wnum].y+windows[wnum or _wnum].height)) / temp
window_chg_atts[wnum or _wnum][12][4][3] = temp

temp = math.max(1,math.floor(distance(tempx+_addx,tempy+_addy,windows[wnum or _wnum].x+windows[wnum or _wnum].width,windows[wnum or _wnum].y+windows[wnum or _wnum].height)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][7] = ((tempx+_addx) - (windows[wnum or _wnum].x+windows[wnum or _wnum].width)) / temp --bottomright
window_chg_atts[wnum or _wnum][12][2][8] = ((tempy+_addy) - (windows[wnum or _wnum].y+windows[wnum or _wnum].height)) / temp
window_chg_atts[wnum or _wnum][12][4][4] = temp

window_chg_atts[wnum or _wnum][12][2][9] = windows[wnum or _wnum].x --gotta specify it
window_chg_atts[wnum or _wnum][12][2][10] = windows[wnum or _wnum].y
window_chg_atts[wnum or _wnum][12][3] = 0 --current counter
table.insert(statusbar_objects,{"win",windows[wnum or _wnum].title,windows[wnum or _wnum].id,LuaX_default_icon,LuaX_default_thumb,window_chg_atts[wnum or _wnum][12][5]})
statusbar_redraw = true
end

function LuaX_maximizewindow(wnum,currx_)
window_chg_atts[wnum or _wnum][12][1] = "maxing"
currx = currx or currx_
tempx = currx
tempy = (272-statusbar_2_height) + (statusbar_2_height/2-statusbar_2_icondim/2)
_addx = statusbar_2_icondim-1
_addy = statusbar_2_icondim-1
if statusbar_type == 1 then --modify for start bar
	tempy = 272-statusbar_1_height+statusbar_1_y+4
	_addx = 79
	_addy = 13
end

--copying code ftw!
local temp = math.max(1,math.floor(distance(tempx,tempy,windows[wnum or _wnum].x,windows[wnum or _wnum].y-window_barsize)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][1] = -((tempx - windows[wnum or _wnum].x) / temp) --topleft
window_chg_atts[wnum or _wnum][12][2][2] = -((tempy - (windows[wnum or _wnum].y-window_barsize)) / temp)
window_chg_atts[wnum or _wnum][12][4][1] = temp --max loops

temp = math.max(1,math.floor(distance(tempx+_addx,tempy,windows[wnum or _wnum].x+windows[wnum or _wnum].width,windows[wnum or _wnum].y-window_barsize)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][3] = -(((tempx+_addx) - (windows[wnum or _wnum].x+windows[wnum or _wnum].width)) / temp) --topright
window_chg_atts[wnum or _wnum][12][2][4] = -((tempy - (windows[wnum or _wnum].y-window_barsize)) / temp)
window_chg_atts[wnum or _wnum][12][4][2] = temp

temp = math.max(1,math.floor(distance(tempx,tempy+_addy,windows[wnum or _wnum].x,windows[wnum or _wnum].y+windows[wnum or _wnum].height)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][5] = -((tempx - windows[wnum or _wnum].x) / temp) --bottomleft
window_chg_atts[wnum or _wnum][12][2][6] = -(((tempy+_addy) - (windows[wnum or _wnum].y+windows[wnum or _wnum].height)) / temp)
window_chg_atts[wnum or _wnum][12][4][3] = temp

temp = math.max(1,math.floor(distance(tempx+_addx,tempy+_addy,windows[wnum or _wnum].x+windows[wnum or _wnum].width,windows[wnum or _wnum].y+windows[wnum or _wnum].height)/window_min_num))
window_chg_atts[wnum or _wnum][12][2][7] = -(((tempx+_addx) - (windows[wnum or _wnum].x+windows[wnum or _wnum].width)) / temp) --bottomright
window_chg_atts[wnum or _wnum][12][2][8] = -(((tempy+_addy) - (windows[wnum or _wnum].y+windows[wnum or _wnum].height)) / temp)
window_chg_atts[wnum or _wnum][12][4][4] = temp

window_chg_atts[wnum or _wnum][12][2][9] = tempx --gotta specify it
window_chg_atts[wnum or _wnum][12][2][10] = tempy

window_chg_atts[wnum or _wnum][12][3] = 0 --current counter

for a=1,table.getn(statusbar_objects) do
	if statusbar_objects[a][1] == "win" and statusbar_objects[a][6] == window_chg_atts[wnum or _wnum][12][5] then
		table.remove(statusbar_objects,a)
		break
	end
end
	a=wnum or _wnum
	if a > 1 then
		table.insert(windows,1,windows[a])
		table.remove(windows,a+1)
		table.insert(window_co,1,window_co[a])
		table.remove(window_co,a+1)
		table.insert(window_imgs,1,window_imgs[a])
		table.remove(window_imgs,a+1)
		table.insert(window_chg_atts,1,window_chg_atts[a])
		table.remove(window_chg_atts,a+1)
	end
statusbar_redraw = true
end

function LuaX_getWindowById(wid)
for a=1,table.getn(windows) do
	if windows[a].id == wid then
		return a
	end
end
end

function LuaX_redrawwindow(wnum)
window_chg_atts[wnum or _wnum][9] = true
end

function LuaX_redrawobject(wobj,wnum)
if window_chg_atts[wnum or _wnum][8][wobj] then
window_chg_atts[wnum or _wnum][8][wobj].redraw = true
end
end

function LuaX_addwindowobject(wobjname,wobj,wnum)
windows[wnum or _wnum][wobjname] = wobj
window_chg_atts[wnum or _wnum][8][wobjname] = {type=wobj.type,x=wobj.x,y=wobj.y,color=wobj.color,value=wobj.value,redraw=true}
--window_chg_atts[wnum or _wnum][9] = true
end

function LuaX_removewindowobject(wobjname,wnum)
windows[wnum or _wnum][wobjname] = nil
window_chg_atts[wnum or _wnum][8][wobjname] = nil
LuaX_redrawwindow(wnum or _wnum)
end

function LuaX_mouseOverWindow(wnum)
return (_LuaX_windowOver == (wnum or _wnum))
end

function LuaX_mouseOverObject(objname,wnum)
if (_LuaX_windowOver == (wnum or _wnum)) == false then return false end
wnum = wnum or _wnum
if windows[wnum][objname] == nil then return false end
	if ((windows[wnum][objname].type == "button" or windows[wnum][objname].type == "textbox") and cursor.x-windows[wnum].x >= windows[wnum][objname].x and cursor.x-windows[wnum].x <= windows[wnum][objname].x+string.len(windows[wnum][objname].value)*8+4
		and cursor.y-windows[wnum].y >= windows[wnum][objname].y and cursor.y-windows[wnum].y <= windows[wnum][objname].y+8+4) or
		(windows[wnum][objname].type == "text" and cursor.x-windows[wnum].x >= windows[wnum][objname].x and cursor.x-windows[wnum].x <= windows[wnum][objname].x+string.len(windows[wnum][objname].value)*8
		and cursor.y-windows[wnum].y >= windows[wnum][objname].y and cursor.y-windows[wnum].y <= windows[wnum][objname].y+8) or
		(windows[wnum][objname].type == "image" and cursor.x-windows[wnum].x >= windows[wnum][objname].x and cursor.x-windows[wnum].x <= windows[wnum][objname].x+windows[wnum][objname].value:width()
		and cursor.y-windows[wnum].y >= windows[wnum][objname].y and cursor.y-windows[wnum].y <= windows[wnum][objname].y+windows[wnum][objname].value:height())
		then
			return true
	end
return false
end

local function _LuaX_window_1(x) --create window guts
window_imgs[x]:clear(window.bgcolor)
end

local function _LuaX_window_1_1(x) --create border
if window.barstyle == 0 then
tempc = window.barcolor
elseif window.barstyle == 1 then
tempc = makecolordarker(window.barcolor,color_change,true)
else
return
end
if window.border > 0 then
	window_imgs[x]:fillRect(0,window_barsize,window.border,window.height,tempc)
	window_imgs[x]:fillRect(window.width-window.border,window_barsize,window.border,window.height,tempc)
	window_imgs[x]:fillRect(1,window.height+window_barsize-window.border,window.width-1,window.border,tempc)
end
end

local function _LuaX_window_2(x) --create window title
if window.barstyle == 0 then
window_imgs[x]:fillRect(0,0,window.width,window_barsize,window.barcolor)
elseif window.barstyle == 1 then
local topcolor = makecolorlighter(window.barcolor,color_change,true)
topcolor = topcolor:colors()
local bottomcolor = makecolordarker(window.barcolor,color_change,true)
bottomcolor = bottomcolor:colors()
if window_barsize ~= 0 then
local tempr = (topcolor.r - bottomcolor.r)/math.max(1,window_barsize-1) --make sure it cannot be 0
local tempg = (topcolor.g - bottomcolor.g)/math.max(1,window_barsize-1)
local tempb = (topcolor.b - bottomcolor.b)/math.max(1,window_barsize-1)
for temp = 0, window_barsize-1 do
	if temp == 0 then
		window_imgs[x]:drawLine(1,0,window.width-2,0,Color.new(topcolor.r-tempr*temp,topcolor.g-tempg*temp,topcolor.b-tempb*temp,topcolor.a))
	else
		window_imgs[x]:drawLine(0,temp,window.width-1,temp,Color.new(topcolor.r-tempr*temp,topcolor.g-tempg*temp,topcolor.b-tempb*temp,topcolor.a))
	end
end
end
end
window_imgs[x]:pixel(0,0,Color.new(0,0,0,0))
window_imgs[x]:pixel(window.width-1,0,Color.new(0,0,0,0))
window_imgs[x]:blit(window_barsize/2-LuaX_default_icon:height()/2,window_barsize/2-LuaX_default_icon:height()/2,LuaX_default_icon)
window_imgs[x]:print(4+window_barsize/2-LuaX_default_icon:height()/2+LuaX_default_icon:height(),window_barsize/2-4,window.title)
window_imgs[x]:fillRect(window.width-10-(window_barsize/2-5),window_barsize/2-5,10,10,Color.new(200,200,200))
window_imgs[x]:fillRect(window.width-10-(window_barsize/2-5)-10-(window_barsize/2-5),window_barsize/2-5,10,7,Color.new(200,200,200))
window_imgs[x]:fillRect(window.width-10-(window_barsize/2-5)-10-(window_barsize/2-5),window_barsize/2-5+7,10,3,Color.new(100,100,100))

end

function LuaX_desktop_addIcon(icon_name, icon_icon, icon_onClick, iconx, icony)
local icon_x = iconx or 10 --CHANGE
local icon_y = icony or 10 --CHANGE
table.insert(LuaX_desktop_icons,{icon_name, icon_icon, icon_onClick, icon_x, icon_y})
LuaX_desktop_redraw = true
end

function LuaX_desktop_process()
if LuaX_desktop_img == nil or LuaX_desktop_redraw then
	if LuaX_desktop_img == nil then
		LuaX_desktop_img = Image.createEmpty(480,272)
	end
	LuaX_desktop_img:blit(0,0,_bg)
	for a=table.getn(LuaX_desktop_icons),1,-1 do
		LuaX_desktop_img:blit(LuaX_desktop_icons[a][4],LuaX_desktop_icons[a][5],LuaX_desktop_icons[a][2])
		local str = string.sub(LuaX_desktop_icons[a][1],1,10)
		local tempx = LuaX_desktop_icons[a][4]+statusbar_2_icondim/2-string.len(str)*4
		local tempx2 = 0
		if tempx < 0 then
			tempx2 = math.ceil(-tempx/8)
			tempx = tempx + tempx2*8
		end
		_LuaX_ct(LuaX_desktop_img,tempx,LuaX_desktop_icons[a][5]+statusbar_2_icondim+3,string.sub(str,tempx2+1))
	end
	LuaX_desktop_redraw = false
end
end


function LuaX_desktop_render()
_LuaX_screen:blit(0,0,LuaX_desktop_img,false)
end











































function LuaX_windows_process()

-- mouse control
if math.abs(input:analogX()) > 30 then
	c_movx = c_movx + (input:analogX()* (60/fps))^2/80000 * (input:analogX()/math.abs(input:analogX()))
	--[[if c_movx > 5 then c_movx = 5 end
	if c_movx < -5 then c_movx = -5 end]]
else
	c_movx = c_movx / cursor_f
end
if math.abs(input:analogY()) > 30 then
	c_movy = c_movy + (input:analogY()* (60/fps))^2/80000 * (input:analogY()/math.abs(input:analogY()))
	--[[if c_movy > 5 then c_movy = 5 end
	if c_movy < -5 then c_movy = -5 end]]
else
	c_movy = c_movy / cursor_f
end
cursor.x = cursor.x + c_movx 
cursor.y = cursor.y + c_movy
if cursor.x < 0 then
	cursor.x = 0
	c_movx = -(c_movx / 3)
elseif cursor.x > 479 then
	cursor.x = 479
	c_movx = -(c_movx / 3)
end
if cursor.y < 0 then
	cursor.y = 0
	c_movy = -(c_movy / 3)
elseif cursor.y > 271 then
	cursor.y = 271
	c_movy = -(c_movy / 3)
end
-- /mouse control

if LuaX_state ~= "none" then return end

if cursor.visible then
if LuaX_desktop_dclickcnt > 0 then LuaX_desktop_dclickcnt = LuaX_desktop_dclickcnt + 1; if LuaX_desktop_dclickcnt > LuaX_desktop_dclickcnt_max then LuaX_desktop_dclickcnt = 0 end end
_LuaX_windowOver = getwindowclick()
	if cursor.state == "none" and input:cross() and not oldinput:cross() then
		a = _LuaX_windowOver
		screen:print(0,10,tostring(a))
			if a and a >= 1 then
				if a > 1 then
				table.insert(windows,1,windows[a])
				table.remove(windows,a+1)
				table.insert(window_co,1,window_co[a])
				table.remove(window_co,a+1)
				table.insert(window_imgs,1,window_imgs[a])
				table.remove(window_imgs,a+1)
				table.insert(window_chg_atts,1,window_chg_atts[a])
				table.remove(window_chg_atts,a+1)
				end
				
				window = windows[1]
				screen:print(0,20,'ouch!')
				if cursor.y <= window.y then
					--check for button presses
					if cursor.x - window.x >= window.width-10-(window_barsize/2-5) and cursor.x - window.x <= (window.width-10-(window_barsize/2-5))+10
						and cursor.y - (window.y - window_barsize) >= window_barsize/2-5 and cursor.y - (window.y - window_barsize) <= (window_barsize/2-5)+10
						then --close
							LuaX_closewindow(1)
							cursor.state = "none"
					elseif cursor.x - window.x >= window.width-10-(window_barsize/2-5)-10-(window_barsize/2-5) and cursor.x - window.x <= (window.width-10-(window_barsize/2-5)-10-(window_barsize/2-5))+10
						and cursor.y - (window.y - window_barsize) >= window_barsize/2-5 and cursor.y - (window.y - window_barsize) <= (window_barsize/2-5)+10
						then --minimize
							LuaX_minimizewindow(1)
							cursor.state = "none"
					else
						cursor.state = "move"
						old_cursorx = cursor.x
						old_cursory = cursor.y
						screen:print(0,30,"1")
					end
				else --clicked part of the window content, try to find the object it hit
					closedwindow = false
					table.foreach(window,function(b,c)
											if type(window[b]) == "table" and closedwindow == false then
												if ((window[b].type == "button" or window[b].type == "textbox") and cursor.x-window.x >= window[b].x and cursor.x-window.x <= window[b].x+string.len(window[b].value)*8+4
																				and cursor.y-window.y >= window[b].y and cursor.y-window.y <= window[b].y+8+4) or
												   (window[b].type == "text" and cursor.x-window.x >= window[b].x and cursor.x-window.x <= window[b].x+string.len(window[b].value)*8
																			   and cursor.y-window.y >= window[b].y and cursor.y-window.y <= window[b].y+8) or
												   (window[b].type == "image" and cursor.x-window.x >= window[b].x and cursor.x-window.x <= window[b].x+window[b].value:width()
																			   and cursor.y-window.y >= window[b].y and cursor.y-window.y <= window[b].y+window[b].value:height())
													then
														if type(window[b].onClick) == "function" then
															_wnum = 1
															window[b].onClick()
														end
														screen:print(0,30,b)
												end
											end
										end )
				end
			elseif a and a == 0 then
				screen:print(0,20,"you clicked the desktop")
				local iconclick = false
				for b=1,table.getn(LuaX_desktop_icons) do
					if cursor.x >= LuaX_desktop_icons[b][4] and cursor.x <= LuaX_desktop_icons[b][4] + statusbar_2_icondim and cursor.y >= LuaX_desktop_icons[b][5] and cursor.y <= LuaX_desktop_icons[b][5] + statusbar_2_icondim then
						iconclick = b
						break
					end
				end
				if iconclick then
					if iconclick > 1 then
						table.insert(LuaX_desktop_icons,1,LuaX_desktop_icons[iconclick])
						table.remove(LuaX_desktop_icons,iconclick+1)
						LuaX_desktop_redraw = true
						LuaX_desktop_dclickcnt = 0
						cursor.state = "ichd" --"icon hold"
					else
						if LuaX_desktop_dclickcnt > 0 then
							pcall(LuaX_desktop_icons[1][3])
							LuaX_desktop_dclickcnt = 0
						else
							cursor.state = "ichd" --"icon hold"
						end
					end
					if cursor.state == "ichd" then
						LuaX_desktop_icons[1][6] = cursor.x - LuaX_desktop_icons[1][4]
						LuaX_desktop_icons[1][7] = cursor.y - LuaX_desktop_icons[1][5]
					end
					screen:print(0,50,"you clicked icon: "..LuaX_desktop_icons[1][1])
				end
			end
	elseif cursor.state == "move" then
		if input:cross() then

			windows[1].x = windows[1].x + (cursor.x - old_cursorx)
			windows[1].y = windows[1].y + (cursor.y - old_cursory)
			old_cursorx = cursor.x
			old_cursory = cursor.y
		else
			cursor.state = "none"
			window_chg_atts[1][6] = (cursor.x - old_cursorx)
			window_chg_atts[1][7] = (cursor.y - old_cursory)
		end
	elseif cursor.state == "ichd" then --"icon hold"
		if not input:cross() then
			cursor.state = "none"
			LuaX_desktop_dclickcnt = 1
		elseif math.abs(input:analogX()) > 30 or math.abs(input:analogY()) > 30 then
			cursor.state = "icmv" --"icon move"
		end
	elseif cursor.state == "icmv" then --"icon move"
		if not input:cross() then
			cursor.state = "none"
			LuaX_desktop_icons[1][4] = cursor.x - LuaX_desktop_icons[1][6]
			LuaX_desktop_icons[1][5] = cursor.y - LuaX_desktop_icons[1][7]
			LuaX_desktop_redraw = true
		end
	end
end
	
end --function LuaX_windows_process

function LuaX_windows_render()
--different render, alt-tab
if LuaX_state == "altb" then
	if LuaX_windtab_fade < 150 then LuaX_windtab_fade = LuaX_windtab_fade + 10 end
	temp = {{Color.new(0,0,0,LuaX_windtab_fade),480,272,0},
			{Color.new(0,0,0,LuaX_windtab_fade),0,272,0},
			{Color.new(0,0,0,LuaX_windtab_fade),480,0,0},
			{Color.new(0,0,0,LuaX_windtab_fade),0,0,0}
		   }
		Gu.start3d()
		Gu.disable(Gu.TEXTURE_2D)
		Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.COLOR_8888+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
	if LuaX_windtab_type == 1 then --mac?
			Gu.enable(Gu.TEXTURE_2D)
			Gu.texImage(_bg)
			temp = {{400,186,400,186,0},
					{80,186,80,186,0},
					{400,86,400,86,0},
					{80,86,80,86,0}
				   }
			--[[
			temp:
				MAX,MAX
				min,MAX
				MAX,min
				min,min
			]]
			Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
			--draw icons
			Gu.end3d()
	elseif LuaX_windtab_type == 2 then --flip3d
		Gu.enable(Gu.TEXTURE_2D)
		Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
		local incx = 183 / (table.getn(windows) + 1)
		local incy = 188 / (table.getn(windows) + 1)
		local currx = 1
		local curry = 1
		if LuaX_windtab_d ~= 0 then
			LuaX_windtab_i = LuaX_windtab_i + incx/5 * LuaX_windtab_d
			if math.abs(LuaX_windtab_i) >= math.abs(incx) then
				LuaX_windtab_done = 1
				--LuaX_windtab_i = incx
			end
		end
		for a=table.getn(windows),1,-1 do
			local _width = window_imgs[a]:width()
			local _height = window_imgs[a]:height()
			local width = (_width / 1.5) - (a  )
			local height = (_height / 1.5) - (a  )
			local minx = (incx * currx) + LuaX_windtab_i
			local miny = (incy * curry) + 83 + LuaX_windtab_i * (incy/incx)
			local c = Color.new(255,255,255)
			if a == 1 and LuaX_windtab_d == 1 then
				c = Color.new(255,255,255,255-(LuaX_windtab_i/incx)*255)
			elseif currx == 1 and LuaX_windtab_d == -1 then
				c = Color.new(255,255,255,255-(LuaX_windtab_i/incx)*-255)
			end
			local temp = {{_width,_height,c,minx+width,miny-height/10,0},
						  {0,_height,c,minx,miny,0},
						  {_width,0,c,minx+width,miny-height+height/20,0},
						  {0,0,c,minx,miny-height,0}}
			Gu.texImage(window_imgs[a])
			Gu.texFunc(Gu.TFX_MODULATE,Gu.TCC_RGBA)
			Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.COLOR_8888+Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
			currx = currx + 1
			curry = curry + 1
		end
		Gu.end3d()
	end
return
end
















--render normally
for a=table.getn(windows),1,-1 do
	window = windows[a]
	
	if a > 1 or (a==1 and cursor.state ~= "move") then
		window.x = window.x + window_chg_atts[a][6]
		window.y = window.y + window_chg_atts[a][7]
		window_chg_atts[a][6] = window_chg_atts[a][6] / window_f
		window_chg_atts[a][7] = window_chg_atts[a][7] / window_f
		if window.x+window.width < 0 then
			window.x = -window.width
			window_chg_atts[a][6] = -(window_chg_atts[a][6] / 2)
		elseif window.x > 479 then
			window.x = 479
			window_chg_atts[a][6] = -(window_chg_atts[a][6] / 2)
		end
		if window.y < 0 then
			window.y = 0
			window_chg_atts[a][7] = -(window_chg_atts[a][7] / 2)
		elseif window.y-window_barsize > 271 then
			window.y = 271+window_barsize
			window_chg_atts[a][7] = -(window_chg_atts[a][7] / 2)
		end
	end
	
	if window_chg_atts[a][12][1] == "mining" or window_chg_atts[a][12][1] == "maxing" then
	
		--screen:blit(,window_imgs[a])
		window_chg_atts[a][12][3] = window_chg_atts[a][12][3] + 1 * (60/fps)
		local temp = {}
		_addx = statusbar_2_icondim-1
		_addy = statusbar_2_icondim-1
		if statusbar_type == 1 then --modify for start menu
			_addx = 79
			_addy = 13
		end
		if window_chg_atts[a][12][1] == "mining" then
		 temp = {{0,0,window_chg_atts[a][12][2][9] + window_chg_atts[a][12][2][1]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][1]),(window_chg_atts[a][12][2][10]-window_barsize) + window_chg_atts[a][12][2][2]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][1]),0},
					  {window.width,0,window_chg_atts[a][12][2][9] + window.width + window_chg_atts[a][12][2][3]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][2]),(window_chg_atts[a][12][2][10]-window_barsize) + window_chg_atts[a][12][2][4]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][2]),0},
					  {0,window.height+window_barsize,window_chg_atts[a][12][2][9] + window_chg_atts[a][12][2][5]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][3]),(window_chg_atts[a][12][2][10]-0) + window.height + window_chg_atts[a][12][2][6]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][3]),0},
					  {window.width,window.height+window_barsize,window_chg_atts[a][12][2][9] + window.width + window_chg_atts[a][12][2][7]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][4]),(window_chg_atts[a][12][2][10]+0) + window.height + window_chg_atts[a][12][2][8]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][4]),0}}
		else
		 temp = {{0,0,window_chg_atts[a][12][2][9] + window_chg_atts[a][12][2][1]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][1]),(window_chg_atts[a][12][2][10]) + window_chg_atts[a][12][2][2]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][1]),0},
					  {window.width,0,window_chg_atts[a][12][2][9] + _addx + window_chg_atts[a][12][2][3]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][2]),(window_chg_atts[a][12][2][10]) + window_chg_atts[a][12][2][4]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][2]),0},
					  {0,window.height+window_barsize,window_chg_atts[a][12][2][9] + window_chg_atts[a][12][2][5]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][3]),(window_chg_atts[a][12][2][10]-0) + _addy + window_chg_atts[a][12][2][6]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][3]),0},
					  {window.width,window.height+window_barsize,window_chg_atts[a][12][2][9] + _addx + window_chg_atts[a][12][2][7]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][4]),(window_chg_atts[a][12][2][10]+0) + _addy + window_chg_atts[a][12][2][8]*math.min(window_chg_atts[a][12][3],window_chg_atts[a][12][4][4]),0}}
		end
			Gu.start3d()

          Gu.enable(Gu.TEXTURE_2D)
          Gu.texImage(window_imgs[a])
          Gu.texFilter(Gu.LINEAR, Gu.LINEAR)

          Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
          Gu.end3d()
        if math.max(window_chg_atts[a][12][4][1],window_chg_atts[a][12][4][2],window_chg_atts[a][12][4][3],window_chg_atts[a][12][4][4]) <= window_chg_atts[a][12][3] then
			if window_chg_atts[a][12][1] == "mining" then
				window_chg_atts[a][12][1] = "min"
			else
				window_chg_atts[a][12][1] = "max"
			end
		end
	elseif window_chg_atts[a][12][1] == "min" then
		
	else
		_LuaX_screen:blit(window.x,window.y-window_barsize,window_imgs[a])
	end

end

--run background function
for a=table.getn(windows),1,-1 do
	window = windows[a]
	if window_chg_atts[a][4] ~= window.func then
		window_chg_atts[a][4] = window.func or 0
		if window.func ~= 0 then
			window_co[a] = coroutine.create(window.func)
		else
			window_co[a] = 0
		end
	end
	closedwindow = false
	if window_co[a] ~= 0 then
		_wnum = a
		coroutine.resume(window_co[a])
		if closedwindow == false then
		if coroutine.status(window_co[a]) == "dead" then
			window_co[a] = coroutine.create(window.func)
		end
		end
	end
	if closedwindow == false then
	object_changed = false --we will go through all objects in the window to detirmine if they have changed
	table.foreach(window,function(b,c)
						  if type(window[b]) == "table" then
							_b = b --globalize.
							table.foreach(window_chg_atts[a][8][b],function(c,d)
																	if c == "redraw" then
																		if window_chg_atts[a][8][_b].redraw then
																			object_changed = true
																		end
																	else
																	if window[_b][c] ~= window_chg_atts[a][8][_b][c] then
																		object_changed = true
																		window_chg_atts[a][8][_b][c] = window[_b][c]
																		window_chg_atts[a][8][_b].redraw = true
																		screen:print(0,0,"object changed")
																	end
																	end
																   end )
						  end
						 end )
	--re-render window if necessary
	if window.width ~= window_chg_atts[a][1] or window.height ~= window_chg_atts[a][2] then
		window_imgs[a] = Image.createEmpty(window.width,window.height+window_barsize)
		_LuaX_window_1(a) --window
		_LuaX_window_1_1(a) --window
		_LuaX_window_2(a) --title
		window_chg_atts[a][1] = window.width
		window_chg_atts[a][2] = window.height
		window_chg_atts[a][3] = window.bgcolor
		window_chg_atts[a][5] = window.title
		window_chg_atts[a][9] = true
		window_chg_atts[a][10] = window.border
		window_chg_atts[a][11] = window.barcolor
	elseif window.bgcolor ~= window_chg_atts[a][3] or window_chg_atts[a][10] ~= window.border or window_chg_atts[a][9] then --only update window content
		_LuaX_window_1(a) --window
		_LuaX_window_1_1(a) --window
		_LuaX_window_2(a) --title
		window_chg_atts[a][3] = window.bgcolor
		window_chg_atts[a][5] = window.title
		window_chg_atts[a][9] = true
		window_chg_atts[a][10] = window.border
		window_chg_atts[a][11] = window.barcolor
	elseif window.title ~= window_chg_atts[a][5] or window.barcolor ~= window_chg_atts[a][11] then --only update bar
		if window.barcolor ~= window_chg_atts[a][11] then
			_LuaX_window_1_1(a) --border
		end
		_LuaX_window_2(a) --title
		window_chg_atts[a][5] = window.title
		window_chg_atts[a][11] = window.barcolor
	end
	
	if object_changed or window_chg_atts[a][9] then
		screen:print(0,10,"redrawingz0r")
		table.foreach(window,function(c,d)
					if type(window[c]) == "table" then
					if window_chg_atts[a][8][c].redraw or window_chg_atts[a][9] then
						window_chg_atts[a][8][c].redraw = false
						if string.lower(window[c].type) == "button" then
							window_imgs[a]:fillRect(window[c].x+1,window[c].y+window_barsize+1,string.len(window[c].value)*8+4-2,8+4-2,window[c].color)
							window_imgs[a]:drawLine(window[c].x+1,window[c].y+window_barsize,window[c].x+string.len(window[c].value)*8+4-1-1,window[c].y+window_barsize,makecolorlighter(window[c].color,color_change)) --smooth button top
							window_imgs[a]:drawLine(window[c].x,window[c].y+window_barsize+1,window[c].x,window[c].y+window_barsize+8+4-1-1,makecolorlighter(window[c].color,color_change)) --left
							window_imgs[a]:drawLine(window[c].x+1,window[c].y+window_barsize+8+4-1,window[c].x+string.len(window[c].value)*8+4-1-1,window[c].y+window_barsize+8+4-1,makecolordarker(window[c].color,color_change)) --bottom
							window_imgs[a]:drawLine(window[c].x+string.len(window[c].value)*8+4-1,window[c].y+window_barsize+1,window[c].x+string.len(window[c].value)*8+4-1,window[c].y+window_barsize+8+4-1-1,makecolordarker(window[c].color,color_change)) --right
							window_imgs[a]:print(window[c].x+2,window[c].y+window_barsize+2,window[c].value)
						elseif string.lower(window[c].type) == "textbox" then
							window_imgs[a]:fillRect(window[c].x,window[c].y+window_barsize,string.len(window[c].value)*8+4,8+4,Color.new(0,0,0))
							window_imgs[a]:fillRect(window[c].x+1,window[c].y+window_barsize+1,string.len(window[c].value)*8+4-2,8+4-2,window[c].color)
							window_imgs[a]:print(window[c].x+2,window[c].y+window_barsize+2,window[c].value)
						elseif string.lower(window[c].type) == "text" then
							window_imgs[a]:print(window[c].x,window[c].y+window_barsize,window[c].value,window[c].color)
						elseif string.lower(window[c].type) == "image" then
							window_imgs[a]:blit(window[c].x,window[c].y+window_barsize,window[c].value)
						end
					end
					end
				   end)
				   window_chg_atts[a][9] = false
	end
	end --if closedwindow
end

end --function LuaX_windows_render

function LuaX_statusbar_process()
--if LuaX_state ~= "none" and LuaX_state ~= "syst" then return end
if statusbar_img == nil then
	if statusbar_type == 1 then --start bar
		statusbar_img = Image.createEmpty(480,statusbar_1_height)
	elseif statusbar_type == 2 then --mac dock
		statusbar_img = Image.createEmpty(10,statusbar_2_height)
	end
end
if statusbar_redraw then
	statusbar_redraw = false
	if statusbar_type == 1 then --start bar
		statusbar_img:fillRect(0,0,56,20,Color.new(255,150,0))
		statusbar_img:drawLine(0,0,479,0,Color.new(0,0,0))
		statusbar_img:drawLine(0,1,479,1,Color.new(75,75,75))
		--statusbar_img:drawLine(56,2,479,2,Color.new(110,110,110))
		--statusbar_img:drawLine(56,3,479,3,Color.new(72,72,72))
		statusbar_img:drawLine(0,2,479,2,Color.new(37,37,37))
		--statusbar_img:drawLine(56,3,479,3,Color.new(113,113,113))
		statusbar_img:drawLine(51,19,479,19,Color.new(27,27,27))
		statusbar_img:print(7-1,7,"start")
		statusbar_img:print(7+1,7,"start")
		statusbar_img:print(7,7-1,"start")
		statusbar_img:print(7,7+1,"start")
		statusbar_img:print(7,7,"start",Color.new(255,255,255))
		b = 111
		d = 90*3
		for a=3,3+7 do
			if a==3 then
			statusbar_img:drawLine(51+math.cos(math.rad(d))*4,a,479,a,Color.new(b,b,b))
			else
			statusbar_img:drawLine(52+math.cos(math.rad(d))*4,a,479,a,Color.new(b,b,b))
			end
			b=b-3
			d=d+90/8
		end
		b = 61
		d = 1
		for a=3+7+1,3+7+1+7 do
			statusbar_img:drawLine(52+math.cos(math.rad(d))*4,a,479,a,Color.new(b,b,b))
			b=b-2.5
			d=d+90/8
		end
		
		--bars start at pixel 60, are 80 pixels wide seperated by 2 pixels
		currx = 60
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "win" then
				statusbar_img:drawLine(currx+1,4,currx+78,4,Color.new(95,95,95))
				statusbar_img:drawLine(currx,5,currx,16,Color.new(68,68,68))
				statusbar_img:drawLine(currx+79,5,currx+79,16,Color.new(68,68,68))
				statusbar_img:drawLine(currx+1,16,currx+78,16,Color.new(37,37,37))
				statusbar_img:drawLine(currx+1,17,currx+78,17,Color.new(16,16,16))
				b = 116
				for c=5,9 do
					statusbar_img:drawLine(currx+1,c,currx+78,c,Color.new(b,b,b))
					b=b-4.8
				end
				b=76
				for c=10,15 do
					statusbar_img:drawLine(currx+1,c,currx+78,c,Color.new(b,b,b))
					b=b-4
				end
				statusbar_img:drawLine(currx+2,5,currx+76,5,Color.new(125,125,125))
				statusbar_img:blit(currx+3,6,statusbar_objects[a][4])
				statusbar_img:print(currx+3+10+3,8,string.sub(statusbar_objects[a][2],1,8),Color.new(255,255,255))

				currx = currx + 80 + 2
			end
		end
		
		
	elseif statusbar_type == 2 then --mac dock
		local statuswidth = table.getn(statusbar_objects) * (statusbar_2_icondim+10) + 10
		if statuswidth ~= statusbar_img:width() then --recreate bar
			statusbar_img = Image.createEmpty(statuswidth,statusbar_2_height)
		end
		statusbar_img:clear(Color.new(255,255,255,100))
		statusbar_img:drawLine(0,0,0,statusbar_2_height-1,Color.new(100,100,100,150))
		statusbar_img:drawLine(0,0,statuswidth-1,0,Color.new(100,100,100,150))
		statusbar_img:drawLine(statuswidth-1,0,statuswidth-1,statusbar_2_height-1,Color.new(100,100,100,150))
		local currx = 10
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "obj" then
			statusbar_img:blit(currx,(statusbar_2_height/2-statusbar_2_icondim/2),statusbar_objects[a][5])
			currx = currx + statusbar_2_icondim + 10
			end
		end
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "win" then
			statusbar_img:blit(currx,(statusbar_2_height/2-statusbar_2_icondim/2),statusbar_objects[a][5])
			currx = currx + statusbar_2_icondim + 10
			end
		end
	end
end

end --function LuaX_statusbar_process()

function LuaX_statusbar_render()
--if LuaX_state ~= "none" and LuaX_state ~= "syst" then return end
if statusbar_type == 1 then --start bar
	_LuaX_screen:blit(0,272-statusbar_1_height+statusbar_1_y,statusbar_img,false)
	if statusbar_1_active then
		_LuaX_screen:blit(0,272-statusbar_1_height+statusbar_1_y-statusbar_1_menuimg:height()+3,statusbar_1_menuimg,false)
		if cursor.visible and cursor.state == "none" and input:cross() and not oldinput:cross() then
			if cursor.y >= 272-statusbar_1_height+statusbar_1_y-statusbar_1_menuimg:height()+3 and cursor.y <= 272-statusbar_1_height+statusbar_1_y+3 and cursor.x < statusbar_1_menuimg:width() then
			--clicked start menu
				--check item clicked
			end
			 if not (cursor.y > 272-statusbar_1_height+statusbar_1_y+3 and cursor.x <= 54) then --start button, disable start menu
				statusbar_1_active = false
				statusbar_1_menuimg = nil
			end
		end
	end
	if cursor.visible and cursor.state == "none" and input:cross() and oldinput:cross() == false and cursor.y > 272-statusbar_1_height+statusbar_1_y then --check to see if something was clicked
		if cursor.y > 272-statusbar_1_height+statusbar_1_y+3 and cursor.x <= 54 then --start button
			if not statusbar_1_active then --activate start menu
					local total = 0
					for a=1,table.getn(statusbar_objects) do
						if statusbar_objects[a][1] == "obj" then
							total = total + 1
						end
					end
				statusbar_1_menuimg = Image.createEmpty(statusbar_1_menuwidth,math.max(10,total)*11+2+1) --2 begining, 10 each item, 1 spacer, 1 ending (counts in the 11)
				statusbar_1_menuimg:clear(Color.new(111,111,111))
				local a = 0 --currcolor
				local b = 111/(statusbar_1_menuimg:height()-2) --increment
				for c=statusbar_1_menuimg:height()-2,1,-1 do
					statusbar_1_menuimg:drawLine(1,c,10,c,Color.new(a,a,a))
					a = a + b
				end
				statusbar_1_menuimg:drawLine(0,0,statusbar_1_menuimg:width()-1,0,Color.new(0,0,0))
				statusbar_1_menuimg:drawLine(statusbar_1_menuimg:width()-1,0,statusbar_1_menuimg:width()-1,statusbar_1_menuimg:height()-1,Color.new(0,0,0))
				statusbar_1_menuimg:drawLine(statusbar_1_menuimg:width()-1,statusbar_1_menuimg:height()-1,0,statusbar_1_menuimg:height()-1,Color.new(0,0,0))
				statusbar_1_menuimg:drawLine(0,statusbar_1_menuimg:height()-1,0,0,Color.new(0,0,0))
				statusbar_1_menuimg:drawLine(11,1,11,statusbar_1_menuimg:height()-2,Color.new(0,0,0))
				local total = 0
				for a=1,table.getn(statusbar_objects) do
					if statusbar_objects[a][1] == "obj" then
						local b = 200 --currcolor
						local c = (b-111)/(statusbar_1_menuimg:width()-3-13) --decrement
						for d=13,statusbar_1_menuimg:width()-3 do
							statusbar_1_menuimg:drawLine(d,2+total*11,d,2+total*11+9,Color.new(b,b,b))
							b = b - c
						end
						statusbar_1_menuimg:print(15,2+total*11+1,statusbar_objects[a][2])
						total = total + 1
					end
				end
				statusbar_1_active = true
			else --disable it
				statusbar_1_active = false
				statusbar_1_menuimg = nil
			end
		elseif cursor.y >= 272-statusbar_1_height+statusbar_1_y+4 and cursor.y <= 272-statusbar_1_height+statusbar_1_y+17 and cursor.x >= 60 then
			--check each possible button.
			currx = 60
			for a=1,table.getn(statusbar_objects) do
				if statusbar_objects[a][1] == "win" then
					if cursor.x >= currx and cursor.x <= currx+79 then
						--hit button a
						for b=1,table.getn(window_chg_atts) do --match the ID
							if statusbar_objects[a][6] == window_chg_atts[b][12][5] then --maximize window
								LuaX_maximizewindow(b)
								break
							end
						end
						break
					end
					currx = currx + 80 + 2
				end
			end
		end
	end
elseif statusbar_type == 2 then --mac dock
if not statusbar_2_active then --test to see if it should be enabled
	statusbar_2_active = (cursor.x >= 240-statusbar_img:width()/2 and cursor.x <= (240-statusbar_img:width()/2)+statusbar_img:width() and cursor.y >= 272-statusbar_2_height and _LuaX_windowOver == false)
end
	if statusbar_2_active then --create zoomed in
		statuswidth = 10
		local currx = 240-statusbar_img:width()/2+10
		local ypos = (272-statusbar_2_height+(statusbar_2_height/2-statusbar_2_icondim/2)+statusbar_2_icondim)
		local temptable = {}
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "obj" then
				--calculate distance from cursor
					d = math.abs(cursor.x - (currx+statusbar_2_icondim/2))
				--make the icon size
					size = math.max(statusbar_2_icondim,statusbar_2_icondim*statusbar_2_magnify-d/statusbar_2_magnify_sen)
				--create table entry
				table.insert(temptable,{{0,0,(240-statusbar_img:width()/2)+statuswidth,ypos-size,0},
						   {statusbar_objects[a][5]:width(),0,(240-statusbar_img:width()/2)+statuswidth+size,ypos-size,0},
						   {0,statusbar_objects[a][5]:height(),(240-statusbar_img:width()/2)+statuswidth,ypos,0},
					       {statusbar_objects[a][5]:width(),statusbar_objects[a][5]:width(),(240-statusbar_img:width()/2)+statuswidth+size,ypos,0}})
				
				if cursor.x >= (240-statusbar_img:width()/2)+statuswidth and cursor.x <= (240-statusbar_img:width()/2)+statuswidth+size and cursor.y >= ypos-size and cursor.y <= ypos then
				--mouseover
				sprint((240-statusbar_img:width()/2)+statuswidth,ypos-size-10,statusbar_objects[a][2])
					if cursor.visible and cursor.state == "none" and input:cross() and not oldinput:cross() then --check to see if icon was clicked
						screen:print(0,0,statusbar_objects[a][2])
					end
				end
				currx = currx + statusbar_2_icondim + 10
				statuswidth = statuswidth + size + 10
			end
		end
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "win" then
				--calculate distance from cursor
					d = math.abs(cursor.x - (currx+statusbar_2_icondim/2))
				--make the icon size
					size = math.max(statusbar_2_icondim,statusbar_2_icondim*statusbar_2_magnify-d/statusbar_2_magnify_sen)
				--create table entry
				table.insert(temptable,{{0,0,(240-statusbar_img:width()/2)+statuswidth,ypos-size,0},
						{statusbar_objects[a][5]:width(),0,(240-statusbar_img:width()/2)+statuswidth+size,ypos-size,0},
						{0,statusbar_objects[a][5]:height(),(240-statusbar_img:width()/2)+statuswidth,ypos,0},
					    {statusbar_objects[a][5]:width(),statusbar_objects[a][5]:width(),(240-statusbar_img:width()/2)+statuswidth+size,ypos,0}})
				
				if cursor.x >= (240-statusbar_img:width()/2)+statuswidth and cursor.x <= (240-statusbar_img:width()/2)+statuswidth+size and cursor.y >= ypos-size and cursor.y <= ypos then
				--mouseover
				sprint((240-statusbar_img:width()/2)+statuswidth,ypos-size-10,statusbar_objects[a][2])
					if cursor.visible and cursor.state == "none" and input:cross() and not oldinput:cross() then --check to see if icon was clicked
						_ = false
						for b=1,table.getn(window_chg_atts) do --match the ID
							if statusbar_objects[a][6] == window_chg_atts[b][12][5] then --maximize window
								LuaX_maximizewindow(b,(240-statusbar_img:width()/2)+statuswidth)
								_ = true
								break
							end
						end
						if _ then break end
					end
				end
				currx = currx + statusbar_2_icondim + 10
				statuswidth = statuswidth + size + 10
			end
		end
		temp = {{0,0,Color.new(255,255,255,100),240-statusbar_img:width()/2,272-statusbar_2_height,0},
				{statuswidth,0,Color.new(255,255,255,100),240-statusbar_img:width()/2+statuswidth,272-statusbar_2_height,0},
				{0,statusbar_2_height,Color.new(255,255,255,100),240-statusbar_img:width()/2,272,0},
				{statuswidth,statusbar_2_height,Color.new(255,255,255,100),240-statusbar_img:width()/2+statuswidth,272,0}}
				Gu.start3d()
				Gu.disable(Gu.TEXTURE_2D)
				Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D+Gu.COLOR_8888, temp)

		temp = {{Color.new(70,70,70,150),240-statusbar_img:width()/2,272,0},
				{Color.new(70,70,70,150),240-statusbar_img:width()/2,272-statusbar_2_height,0},
				{Color.new(70,70,70,150),240-statusbar_img:width()/2+statuswidth,272-statusbar_2_height,0},
				{Color.new(70,70,70,150),240-statusbar_img:width()/2+statuswidth,272,0}}
				Gu.disable(Gu.TEXTURE_2D)
				Gum.drawArray(Gu.LINE_STRIP, Gu.VERTEX_32BITF+Gu.TRANSFORM_2D+Gu.COLOR_8888, temp)
				
		local temptablei = 1
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "obj" then
				  Gu.enable(Gu.TEXTURE_2D)
				  Gu.texImage(statusbar_objects[a][5])
				  Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
				  Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temptable[temptablei])
				  temptablei = temptablei + 1
			end
		end
		for a=1,table.getn(statusbar_objects) do
			if statusbar_objects[a][1] == "win" then
				  Gu.enable(Gu.TEXTURE_2D)
				  Gu.texImage(statusbar_objects[a][5])
				  Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
				  Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temptable[temptablei])
				  temptablei = temptablei + 1
			end
		end
		Gu.end3d()
	if statusbar_2_active then --test to see if it should be disabled
		statusbar_2_active = (cursor.x >= 240-statusbar_img:width()/2 and cursor.x <= (240-statusbar_img:width()/2)+statuswidth and cursor.y >= 272-statusbar_2_icondim*statusbar_2_magnify)
	end
	
	else
		_LuaX_screen:blit(240-statusbar_img:width()/2,272-statusbar_2_height,statusbar_img)
	end
	--screen:blit(39+10,272-statusbar_2_height+(statusbar_2_height/2-tempimg:height()/2),tempimg)
end
end

statusbar_objects = {} -- name, id, image, onclick

function LuaX_statusbar_addObject(    objname,objid,objicon,objimg,objonclk)
table.insert(statusbar_objects,{"obj",objname,objid,objicon,objimg,objonclk})
statusbar_redraw = true
end

function LuaX_statusbar_removeObject(objid)
for a=1,table.getn(statusbar_objects) do
	if statusbar_objects[a][2] == objid then
		table.remove(statusbar_objects,a)
		statusbar_redraw = true
		break
	end
end
end






function LuaX_cube_process()
if LuaX_cube_state == 0 then
	LuaX_cube_screens = {}
	for a=1,4 do
		LuaX_cube_screens[a] = Image.createEmpty(480,272)
		_LuaX_screen = LuaX_cube_screens[a]
		LuaX_switch_env(a)
		LuaX_desktop_process()
		LuaX_desktop_render()
		
		LuaX_statusbar_process()
		LuaX_windows_process()
		
		if (statusbar_2_active or statusbar_type == 1) and LuaX_state ~= "altb" then
			LuaX_windows_render()
			LuaX_statusbar_render()
		else
			LuaX_statusbar_render()
			LuaX_windows_render()
		end
	end
	LuaX_cube_state = 1
	_LuaX_screen = screen
else
	if math.abs(input:analogX()) > 30 then
		LuaX_cube__rot = input:analogX()^2/40000 * (input:analogX()/math.abs(input:analogX()))
	else
		LuaX_cube__rot = LuaX_cube__rot/1.5
	end
	LuaX_cube_rot = LuaX_cube_rot + LuaX_cube__rot
	LuaX_cube_bgx = LuaX_cube_bgx + LuaX_cube__rot*100
	if LuaX_cube_bgx > 480 then LuaX_cube_bgx = LuaX_cube_bgx - 480 end
	if LuaX_cube_bgx < 0 then LuaX_cube_bgx = LuaX_cube_bgx + 480 end
	
	local screenPoints = {}
	local yx,yz
	
	for i=table.getn(LuaX_cube_pointsArray),1,-1 do

		yz = math.cos(LuaX_cube_rot)*LuaX_cube_pointsArray[i].z - math.sin(LuaX_cube_rot)*LuaX_cube_pointsArray[i].x
		yx = math.sin(LuaX_cube_rot)*LuaX_cube_pointsArray[i].z + math.cos(LuaX_cube_rot)*LuaX_cube_pointsArray[i].x

		screenPoints[i] = {}
		screenPoints[i].x = yx*(LuaX_cube_focalLength/(LuaX_cube_focalLength + yz))+240
		screenPoints[i].y = LuaX_cube_pointsArray[i].y*(LuaX_cube_focalLength/(LuaX_cube_focalLength + yz))+136
	end

	--render
	Gu.start3d()
	Gu.enable(Gu.TEXTURE_2D)
	if screenPoints[2].x > screenPoints[1].x then
	temp = {{480,272,screenPoints[6].x,screenPoints[6].y,0},
			{0,272,screenPoints[5].x,screenPoints[5].y,0},
			{480,0,screenPoints[2].x,screenPoints[2].y,0},
			{0,0,screenPoints[1].x,screenPoints[1].y,0}}
	Gu.texImage(LuaX_cube_screens[1])
	Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
	Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
	end
	if screenPoints[3].x > screenPoints[2].x then
	temp = {{480,272,screenPoints[7].x,screenPoints[7].y,0},
			{0,272,screenPoints[6].x,screenPoints[6].y,0},
			{480,0,screenPoints[3].x,screenPoints[3].y,0},
			{0,0,screenPoints[2].x,screenPoints[2].y,0}}
	Gu.texImage(LuaX_cube_screens[2])
	Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
	Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
	end
	if screenPoints[4].x > screenPoints[3].x then
	temp = {{480,272,screenPoints[8].x,screenPoints[8].y,0},
			{0,272,screenPoints[7].x,screenPoints[7].y,0},
			{480,0,screenPoints[4].x,screenPoints[4].y,0},
			{0,0,screenPoints[3].x,screenPoints[3].y,0}}
	Gu.texImage(LuaX_cube_screens[3])
	Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
	Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
	end
	if screenPoints[1].x > screenPoints[4].x then
	temp = {{480,272,screenPoints[5].x,screenPoints[5].y,0},
			{0,272,screenPoints[8].x,screenPoints[8].y,0},
			{480,0,screenPoints[1].x,screenPoints[1].y,0},
			{0,0,screenPoints[4].x,screenPoints[4].y,0}}
	Gu.texImage(LuaX_cube_screens[4])
	Gu.texFilter(Gu.LINEAR, Gu.LINEAR)
	Gum.drawArray(Gu.TRIANGLE_STRIP, Gu.TEXTURE_32BITF+Gu.VERTEX_32BITF+Gu.TRANSFORM_2D, temp)
	end
	Gu.end3d()
	--/render

	if LuaX_cube_focalLength < 400 then LuaX_cube_focalLength = LuaX_cube_focalLength + 10 end
end
end